"""Copyright (c) 2024, Aydin Abdi.

This module can be used for creating PlantUML and Diagrams from .puml and .py files.
"""

from sys_design_diagram._version import version
from loguru import logger


__author__ = "Aydin Abdi"
__license__ = "MIT"
__version__ = version
